translations={
	'simbolos': (
		('\\\\', 'blindtexlineablindtex', '\n'),
		('\\ ', 'blindtexespacioblindtex', ' '),
		('\\:', 'blindtexespacioblindtex', ' '),
		('\\;', 'blindtexespacioblindtex', ' '),
		('\\,', 'blindtexespacioblindtex', ' '),
		('\\quad', 'blindtexespacioblindtex', ' '),
		('\\qquad', 'blindtexespacioblindtex'*2, ' '),
		('\\%', 'blindtexporcientoblindtex', '%'),
		('\\|', 'blindtexbarraverticalblindtexblindtexbarraverticalblindtex', '||'),
		('|', 'blindtexbarraverticalblindtex', '|'),
		('#', 'blindtexsignonumeroblindtex', '#'),
		('°', 'blindtexgradosblindtex', '°'),
		('\\hdots', 'blindtexhdotsblindtex', '⋯'),
		('\\therefore', 'blindtexthereforeblindtex', '∴'),
		('\\varnothing', 'blindtexvarnothingblindtex', '∅'),
		('\\subseteq', 'blindtexsubseteqblindtex', '⊆'),
		('\\veebar', 'blindtexveebarblindtex', '⊻'),
		('\\nexists', 'blindtexnexistsblindtex', '∄'),
		('\\implies', 'blindteximpliesblindtex', '⇒'),
		('~', 'blindtextildeblindtex', '~'),
		('?', 'blindtexcierrainterrogacionblindtex', '?'),
		('¿', 'blindtexabreinterrogacionblindtex', '¿'),
		('á', 'blindtexatildeblindtex', 'á'),
		('Á', 'blindtexamayusculatildeblindtex', 'Á'),
		('é', 'blindtexetildeblindtex', 'é'),
		('É', 'blindtexemayusculatildeblindtex', 'É'),
		('í', 'blindtexitildeblindtex', 'í'),
		('Í', 'blindteximayusculatildeblindtex', 'Í'),
		('ó', 'blindtexotildeblindtex', 'ó'),
		('Ó', 'blindtexomayusculatildeblindtex', 'Ó'),
		('ú', 'blindtexutildeblindtex', 'ú'),
		('Ú', 'blindtexumayusculatildeblindtex', 'Ú')
	),
	'reemplazos_latex': (
		('\\[', '['),
		('−', '-'),
		('\\lbrack', '['),
		('\\lbrace', '\\{'),
		('\\rbrace', '\\}'),
		('\\rbrack', ']'),
		('\\|', '\\Vert'),
		('\\lvert', '|'),
		('\\Vert', '||'),
		('\\vert', '|'),
		('\\rvert', '|'),
		('\\colon', ':'),
		('\\mapsto', '\\rightarrow'),
		('\\overrightarrow', '\\vec')
	),
	'reemplazos_traduccion': (
		("blindtexpesoblindtex", "$"),
		("-", "−"),
		("blindtexvectorblindtex", "⃗"),
		("^(')", "'"),
		('( ', "("),
		(' )', ")"),
		('{ ', "{"),
		(' }', "}")
	),
	'basura_latex': (
		'\\ ',
		'_{}',
		'^{}',
		'\\pause',
		'\\medskip',
		'\\bigskip',
		'\\smallskip',
		'\\displaystyle',
		'\\limits',
		'\\nolimits',
		'\\rm',
		'\\rm\n',
		'\\Bigg',
		'\\bigg',
		'\\Big',
		'\\mathbb',
		'\\mathrm',
		'\\mathcal',
		'\\big',
		'\n'
	)
}